

#include <sys/types.h>
#include <sys/stat.h>
#include <dlfcn.h>
#include <stdio.h>
#include "unix_File.h"

#define DEBUG

static jfieldID jfid;

typedef int (*sysfunc_t)();

typedef struct syscall {
    char *sym;
    sysfunc_t addr;
} syscall_t;

syscall_t systable[] = {
        { "write", 0 },
        { "writev", 0 },
        { "read", 0 },
        { "readv", 0 },
        { "getmsg", 0 },
        { "putmsg", 0 },
        { "poll", 0 },
        { "open", 0 },
        { "close", 0 },
        { "pipe", 0 },
        { "fcntl", 0},
        { "dup", 0 },
        { "creat", 0 },
        { "accept", 0 },
        { (char *) 0, 0 }
};

#define SYS_OPEN    7

static void
initSyscalls()
{
    syscall_t *tp = systable;
    void *libc;

    void *libjavac; 
    if (libc = dlopen("/home/jdk1.1.1/lib/i586/green_threads/libjava.so",
			    RTLD_LAZY)) {
      while (tp->sym != (char *) 0) {
        tp->addr = (sysfunc_t) dlsym(libc, tp->sym);
        tp++;
      }
    }
}



/*
 * Class:     unix_File
 * Method:    open
 * Signature: (Ljava/lang/String;I)I
 */

JNIEXPORT jint JNICALL Java_unix_File_open__Ljava_lang_String_2I
                                          (JNIEnv *env, 
					   jobject obj, 
					   jstring jpath, 
					   jint oflag)
{
  int status;

  /* get C values from Java params */
  const char *path = (*env) -> GetStringUTFChars(env, jpath, 0);

  /* set file descriptor Java id */
  jclass class = (*env) -> GetObjectClass(env, obj);
  jfid = (*env) -> GetFieldID(env, class, "fd", "I");

  initSyscalls();

# ifdef DEBUG
    fprintf(stderr, "attempting to open %s for access\n", path);
# endif

  /* process */
    status = (*systable[SYS_OPEN].addr)(path, oflag, 0);
  // status = open(path, oflag);

# ifdef DEBUG
    fprintf(stderr, "opened %s on file descriptor %d\n", path, status);
    {char buf[5];
    if (read(status, buf, 4) < 0) perror("open read problem");
    buf[4] = '\0';
    fprintf(stderr, "%s", buf); 
    }
# endif

  /* set Java file descriptor value */
  (*env) -> SetIntField(env, obj, jfid, status);
  
  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jpath, path);

  /* set error? */
  if (status < 0) {
    /* set error */
  }
  return((jint) status);
}

/*
 * Class:     unix_File
 * Method:    open
 * Signature: (Ljava/lang/String;II)I
 */

JNIEXPORT jint JNICALL Java_unix_File_open__Ljava_lang_String_2II
                                          (JNIEnv *env,
					   jobject obj,
					   jstring jpath, 
					   jint oflag, 
					   jint mode)
{

}

/*
 * Class:     unix_File
 * Method:    creat
 * Signature: (Ljava/lang/String;I)I
 */

JNIEXPORT jint JNICALL Java_unix_File_creat(JNIEnv *env,
					    jobject obj,
					    jstring jpath, 
					    jint mode)
{

}

/*
 * Class:     unix_File
 * Method:    link
 * Signature: (Ljava/lang/String;Ljava/lang/String;)I
 */

JNIEXPORT jint JNICALL Java_unix_File_link(JNIEnv *env, 
					   jclass class, 
					   jstring jpath1, 
					   jstring jpath2)
{
  int status;

  /* get C values from Java params */
  const char *path1 = (*env) -> GetStringUTFChars(env, jpath1, 0);
  const char *path2 = (*env) -> GetStringUTFChars(env, jpath2, 0);

# ifdef DEBUG
    fprintf(stderr, "attempting link of %s to %s\n", path1, path2);
# endif

  /* process */
  status = link(path1, path2);  
  
  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jpath1, path1);
  (*env) -> ReleaseStringUTFChars(env, jpath2, path2);

  /* return value or throw exception? */
  if (status == 0) {
    return((jint) 0);
  } else {
    /* set error */
    return((jint) status);
  }
}

/*
 * Class:     unix_File
 * Method:    unlink
 * Signature: (Ljava/lang/String;)I
 */

JNIEXPORT jint JNICALL Java_unix_File_unlink(JNIEnv *env, 
					     jclass class, 
					     jstring jpath)
{
  int status;

  /* get C values from Java params */
  const char *path = (*env) -> GetStringUTFChars(env, jpath, 0);

# ifdef DEBUG
    fprintf(stderr, "attempting unlink of %s\n", path);
# endif

  /* process */
  status = unlink(path);  
  
  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jpath, path);

  /* set error? */
  if (status != 0) {
    /* set error */
  }
  return ((jint) status);

}

/*
 * Class:     unix_File
 * Method:    umask
 * Signature: (I)I
 */

JNIEXPORT jint JNICALL Java_unix_File_umask(JNIEnv *env,
					    jclass class,
					    jint jmask)
{
  /* get C values from Java params */
  mode_t mask = (mode_t) jmask;

# ifdef DEBUG
    fprintf(stderr, "setting umask to %o\n", jmask);
# endif

  /* process */
  mask = umask(mask);

  /* return value */
  return ((jint) mask);
}

/*
 * Class:     unix_File
 * Method:    read
 * Signature: ([B)I
 */
JNIEXPORT jint JNICALL Java_unix_File_read___3B(JNIEnv *env,
					   jobject obj, 
					   jbyteArray buf)
{
  /* get C values from Java params */
  /* Java int's are signed, Posix takes an unsigned for read buf length 
   * No problem converting this direction
   */
  jsize nbyte = (*env) -> GetArrayLength(env, buf);

  return Java_unix_File_read___3BI(env, obj, buf, nbyte);

}

/*
 * Class:     unix_File
 * Method:    read
 * Signature: ([BI)I
 */
JNIEXPORT jint JNICALL Java_unix_File_read___3BI(JNIEnv *env,
					   jobject obj, 
					   jbyteArray jbuf,
					   jint nbyte)
{
  int nread;
  char buf[1024];

  /* get file descriptor
   * this must be set as global by open/create
   */
  int fd = (int) (*env) -> GetIntField(env, obj, jfid);

  /* get a native pointer to the array elmnts */
  // buf = (*env) -> GetByteArrayElements(env, jbuf, NULL);
  
# ifdef DEBUG
    fprintf(stderr, "reading %u bytes from file descriptor %d\n", nbyte, fd);
# endif

  /* process */
  nread = read(fd, buf, nbyte);

# ifdef DEBUG
    fprintf(stderr, "read %d bytes from file descriptor %d\n", nread, fd);
# endif

  /* reset array elements */
  (*env) -> ReleaseByteArrayElements(env, jbuf, buf, 0);
  
  /* return value */
  if (nread == -1) {
    /* error */
    perror("read error");
  }
  return ((jint) nread);
}

/*
 * Class:     unix_File
 * Method:    write
 * Signature: ([B)I
 */

JNIEXPORT jint JNICALL Java_unix_File_write___3B(JNIEnv *env,
						  jobject obj, 
						  jbyteArray buf)
{

}

/*
 * Class:     unix_File
 * Method:    write
 * Signature: ([BI)I
 */

JNIEXPORT jint JNICALL Java_unix_File_write___3BI(JNIEnv *env,
						 jobject obj, 
						 jbyteArray buf,
						 jint nbyte)
{

}

/*
 * Class:     unix_File
 * Method:    close
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_unix_File_close(JNIEnv *env, 
					    jobject obj);
