/**
 * Posix class File.
 *
 * @version     1.0 20/6/98
 * @author      Jan Newmarch.
 * Copyright &copy; Jan Newmarch.
 * Covered by GNU Library General Public License
 */


package posix;

public class File {

    /**
     * read access
     */
    public final static int R_OK;

    /**
     * write access
     */
    public final static int W_OK;

    /**
     * execute access
     */
    public final static int X_OK;

    /**
     * existence
     */
    public final static int F_OK;

    private static int r_ok,
	               w_ok,
	               x_ok,
	               f_ok;

    static {
	initializeConstants(); // sets r_ok, etc

	R_OK = r_ok;
	W_OK = w_ok;
	X_OK = x_ok;
	F_OK = f_ok;
    }

    protected String path = null;

    protected static native void initializeConstants();

    /**
     * Construct a new object from the given file path
     */
    public File(String path) {
	this.path = path;
    }

    /**
     * Return the File pathname
     */
    public String getPath() {
	return path;
    }

    /**
     * Link this file to another.
     * @param path the pathname of the new directory entry.
     * @exception PosixException EACCES, EEXIST, EMLINK,
     * ENAMETOOLONG, ENOENT, ENOSPC, ENOTDIR, EPERM,
     * EROFS, EXDEV.
     */
    public native void link(String path) throws PosixException;

    /**
     * Remove the link to this file.
     * @param path the pathname of the file to be unlinked.
     * @exception PosixException EACCES, EBUSY,
     * ENAMETOOLONG, ENOENT, ENOTDIR, EPERM,
     * EROFS.
     */
    public native void unlink() throws PosixException;

    /**
     * MOVE THIS TO PROCESS
     * Sets the process's file mode creation mask
     * and returns the previous value of the mask.
     * @param mask the new creation mask value
     */
    // public native int umask(int mask);

    /**
     * Rename a file.
     * @exception PosixException EACCES, EBUSY, EEXIST,
     * EINVAL, EISDIR,
     * ENAMETOOLONG, ENOENT, ENOSPC, ENOTDIR,
     * EROFS, EXDEV.
     */
    public native void rename(String newName) throws PosixException;

    /**
     * Check the accessibility of the file.
     * @param mode the bitwise OR of R_OK, W_OK, X_OK
     * @exception PosixException EACCES, ENAMETOOLONG,
     * ENOENT, ENOTDIR, EROFS.
     */
    public native void access(int mode) throws PosixException;

    /**
     * Set the permissions mode of a file.
     * The mode is a bitwise-or of file permissions
     * @see Stat
     * @exception PosixException EACCES, ENAMETOOLONG,
     * ENOENT, ENOTDIR, EPERM, EROFS.
     */
    public native void chmod(int mode) throws PosixException;

    /**
     * Set the ownership of a file.
     * @exception PosixException  EACCES, ENAMETOOLONG,
     * ENOTDIR, EPERM, EROFS.
     */
    public native void chown(int owner, int group) throws PosixException;

}
