


/*
 * Class:     posix_File
 * Method:    open
 * Signature: (Ljava/lang/String;I)I
 */

JNIEXPORT jint JNICALL Java_posix_File_open__Ljava_lang_String_2I
                                          (JNIEnv *env, 
					   jobject obj, 
					   jstring jpath, 
					   jint oflag)
{
  int status;

  /* get C values from Java params */
  const char *path = (*env) -> GetStringUTFChars(env, jpath, 0);

  /* get file descriptor Java id */
  jclass class = (*env) -> GetObjectClass(env, obj);
  jfieldID jfid = (*env) -> GetFieldID(env, class, "fd", "I");

  // initSyscalls();

# ifdef DEBUG
    fprintf(stderr, "attempting to open %s for access\n", path);
# endif

  /* process */
  // fprintf(stderr, "open function: %p\n: ", systable[SYS_OPEN].addr);
  // status = (*systable[SYS_OPEN].addr)(path, oflag, 0);
  status = open(path, oflag);

# ifdef DEBUG
    fprintf(stderr, "opened %s on file descriptor %d with flag %d\n", path, status, oflag);
# endif

  /* set Java file descriptor value */
  (*env) -> SetIntField(env, obj, jfid, status);
  
  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jpath, path);

  /* set error? */
  if (status < 0) {
    /* set error */
  }
  return((jint) status);
}

/*
 * Class:     posix_File
 * Method:    open
 * Signature: (Ljava/lang/String;II)I
 */

JNIEXPORT jint JNICALL Java_posix_File_open__Ljava_lang_String_2II
                                          (JNIEnv *env,
					   jobject obj,
					   jstring jpath, 
					   jint oflag, 
					   jint mode)
{
  int status;

  /* get C values from Java params */
  const char *path = (*env) -> GetStringUTFChars(env, jpath, 0);

  /* get file descriptor Java id */
  jclass class = (*env) -> GetObjectClass(env, obj);
  jfieldID jfid = (*env) -> GetFieldID(env, class, "fd", "I");


# ifdef DEBUG
    fprintf(stderr, "attempting to open %s for access\n", path);
# endif

  /* process */
  status = open(path, oflag, mode);

# ifdef DEBUG
    fprintf(stderr, "opened %s on file descriptor %d with flag %d\n", path, status, oflag);
# endif

  /* set Java file descriptor value */
  (*env) -> SetIntField(env, obj, jfid, status);
  
  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jpath, path);

  /* set error? */
  if (status < 0) {
    /* set error */
  }
  return((jint) status);

}

/*
 * Class:     posix_File
 * Method:    creat
 * Signature: (Ljava/lang/String;I)I
 */

JNIEXPORT jint JNICALL Java_posix_File_creat(JNIEnv *env,
					    jobject obj,
					    jstring jpath, 
					    jint mode)
{
  int status;

  /* get C values from Java params */
  const char *path = (*env) -> GetStringUTFChars(env, jpath, 0);

  /* get file descriptor Java id */
  jclass class = (*env) -> GetObjectClass(env, obj);
  jfieldID jfid = (*env) -> GetFieldID(env, class, "fd", "I");


# ifdef DEBUG
    fprintf(stderr, "attempting to create %s for access\n", path);
# endif

  /* process */
  status = creat(path, mode);

# ifdef DEBUG
    fprintf(stderr, "opened %s on file descriptor %d with flag %d\n", path, status, oflag);
# endif

  /* set Java file descriptor value */
  (*env) -> SetIntField(env, obj, jfid, status);
  
  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jpath, path);

  /* set error? */
  if (status < 0) {
    /* set error */
  }
  return((jint) status);
}

/*
 * Class:     posix_File
 * Method:    read
 * Signature: ([B)I
 */
JNIEXPORT jint JNICALL Java_posix_File_read___3B(JNIEnv *env,
					   jobject obj, 
					   jbyteArray buf)
{
  /* get C values from Java params */
  /* Java int's are signed, Posix takes an unsigned for read buf length 
   * No problem converting this direction
   */
  jsize nbyte = (*env) -> GetArrayLength(env, buf);

  return Java_posix_File_read___3BI(env, obj, buf, nbyte);

}

/*
 * Class:     posix_File
 * Method:    read
 * Signature: ([BI)I
 */
JNIEXPORT jint JNICALL Java_posix_File_read___3BI(JNIEnv *env,
					   jobject obj, 
					   jbyteArray jbuf,
					   jint nbyte)
{
  int nread;
  jbyte *buf;

  /* get file descriptor Java id */
  jclass class = (*env) -> GetObjectClass(env, obj);
  jfieldID jfid = (*env) -> GetFieldID(env, class, "fd", "I");

  /* get file descriptor */
  int fd = (int) (*env) -> GetIntField(env, obj, jfid);

  /* get a native pointer to the array elmnts */
  buf = (*env) -> GetByteArrayElements(env, jbuf, NULL);
  
# ifdef DEBUG
    fprintf(stderr, "reading %u bytes from file descriptor %d\n", nbyte, fd);
# endif

  /* process */
  nread = read(fd, buf, nbyte);

# ifdef DEBUG
    fprintf(stderr, "read %d bytes from file descriptor %d\n", nread, fd);
# endif

  /* reset array elements */
  (*env) -> ReleaseByteArrayElements(env, jbuf, buf, 0);
  
  /* return value */
  if (nread == -1) {
    /* error */
    perror("read error");
  }
  return ((jint) nread);
}

/*
 * Class:     posix_File
 * Method:    write
 * Signature: ([B)I
 */

JNIEXPORT jint JNICALL Java_posix_File_write___3B(JNIEnv *env,
						  jobject obj, 
						  jbyteArray buf)
{
  /* get C values from Java params */
  /* Java int's are signed, Posix takes an unsigned for read buf length 
   * No problem converting this direction
   */
  jsize nbyte = (*env) -> GetArrayLength(env, buf);

  return Java_posix_File_write___3BI(env, obj, buf, nbyte);

}

/*
 * Class:     posix_File
 * Method:    write
 * Signature: ([BI)I
 */

JNIEXPORT jint JNICALL Java_posix_File_write___3BI(JNIEnv *env,
						 jobject obj, 
						 jbyteArray jbuf,
						 jint nbyte)
{
  int nwrote;
  jbyte *buf;

  /* get file descriptor Java id */
  jclass class = (*env) -> GetObjectClass(env, obj);
  jfieldID jfid = (*env) -> GetFieldID(env, class, "fd", "I");

  /* get file descriptor */
  int fd = (int) (*env) -> GetIntField(env, obj, jfid);

  /* get a native pointer to the array elmnts */
  buf = (*env) -> GetByteArrayElements(env, jbuf, NULL);
  
# ifdef DEBUG
    fprintf(stderr, "writing %u bytes to file descriptor %d\n", nbyte, fd);
# endif

  /* process */
  nwrote = write(fd, buf, nbyte);

# ifdef DEBUG
    fprintf(stderr, "wrote %d bytes to file descriptor %d\n", nread, fd);
# endif

  /* reset array elements */
  (*env) -> ReleaseByteArrayElements(env, jbuf, buf, 0);
  
  /* return value */
  if (nwrote == -1) {
    /* error */
    perror("read error");
  }
  return ((jint) nwrote);

}

/*
 * Class:     posix_File
 * Method:    close
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_posix_File_close(JNIEnv *env, 
					    jobject obj)
{
  int status;

  /* get file descriptor Java id */
  jclass class = (*env) -> GetObjectClass(env, obj);
  jfieldID jfid = (*env) -> GetFieldID(env, class, "fd", "I");

  /* get file descriptor */
  int fd = (int) (*env) -> GetIntField(env, obj, jfid);

  /* process */
  status = close(fd);

  /* set error? */
  if (status != 0) {
    /* set error */
  }
  return ((jint) status);
}
