/**
 * Posix class OpenFile.
 *
 * @version     1.0 20/6/98
 * @author      Jan Newmarch.
 * Copyright &copy; Jan Newmarch.
 * Covered by GNU Library General Public License
 */


package posix;

public class OpenFile {

    protected String path;

    int fd = -1;

    /**
     * Open for read only, O_RDONLY
     */
    public final static int RDONLY;

    /**
     * Open for write only, O_WRONLY
     */
    public final static int WRONLY;

    /**
     * Oprn for read/write, O_RDWR
     */
    public final static int RDWR;

    /**
     * Open for append, O_APPEND
     */
    public final static int APPEND;

    /**
     * Create file if it doesn't exist, O_CREAT
     */
    public final static int CREAT;

    /**
     * Exclusive use flag, O_EXCL
     */
    public final static int EXCL;

    /**
     * Don't assigna controlling terminal, O_NOCTTY
     */
    public final static int NOCTTY;

    /**
     * No delay, O_NONBLOCK
     */
    public final static int NONBLOCK;

    /**
     * Truncate flag, O_TRUNC
     */
    public final static int TRUNC;

    private static int o_rdonly,
	               o_wronly,
	               o_rdwr,
	               o_append,
	               o_creat,
	               o_excl,
	               o_noctty,
	               o_nonblock,
	               o_trunc;
	               
    static {
        initializeConstants(); // sets o_rdonly, etc

	RDONLY   = o_rdonly;
	WRONLY   = o_wronly;
	RDWR     = o_rdwr;
	APPEND   = o_append;
	CREAT    = o_creat;
	EXCL     = o_excl;
	NOCTTY   = o_noctty;
	NONBLOCK = o_nonblock;
	TRUNC    = o_trunc;
    }

    protected static native void initializeConstants();

    /**
     * Default constructor is only available within
     * this package, for stdin, stdout, stderr
     */
    OpenFile() {
	// empty
    }

    public OpenFile(String path) {
	this.path = path;
    }

    public String getPath() {
	return path;
    }


    /**
     * Opens the file with the access specified.
     * @param path the pathname of the file
     * @param oflag the access mode to open the file
     * @exception PosixException EACCESS, EEXIST, EINTR,
     * EISDIR, EMFILE, ENAMETOOLONG, ENFILE, ENOENT,
     * ENOSPEC, ENOTDIR, ENXIOM EROFS.
     */
    native void open(String path, int oflag) 
	throws PosixException;

    /**
     * Opens the file with the access specified.
     * @param path the pathname of the file
     * @param oflag the access mode to open the file
     * @param mode file permission bits
     * @exception PosixException EACCESS, EEXIST, EINTR,
     * EISDIR, EMFILE, ENAMETOOLONG, ENFILE, ENOENT,
     * ENOSPEC, ENOTDIR, ENXIOM EROFS.
     */
    public native void open(String path, int oflag, int mode)
	throws PosixException;

    /**
     * Close the file
     * @exception PosixException EBADF, EINTR
     */
    public native void close() throws PosixException;

    /**
     * Create the file.
     * Is equivalent to 
     * open(path, (WRONLY | CREAT | TRUNC), oflag)
     */
    public native void creat(String path, int oflag)
	throws PosixException;

    /**
     * dup the file descriptor
     * @exception PosixException EBADF, EMFILE
     */
    protected native void dup2(int fd)
	throws PosixException;

    /**
     * dup the file descriptor
     * @exception PosixException EBADF, EMFILE
     */
     public void dup2(OpenFile f)
	throws PosixException {
	// do some class checking here first?
	dup2(f.getFd());
    }

    /**
     * Return the file descriptor
     */
    public int getFd() {
	return fd;
    }
}
