/**
 * posix_Pipe.
 *
 * @version     1.0 20/6/98
 * @author      Jan Newmarch.
 * Copyright &copy; Jan Newmarch.
 * Covered by GNU Library General Public License
 */


#include "posix_Pipe.h"
#include "PosixException.h"

/* #define DEBUG */

/*
 * Class:     posix_Pipe
 * Method:    pipe
 * Signature: ()[I
 */
JNIEXPORT jint JNICALL Java_posix_Pipe_pipe
  (JNIEnv *env, jobject obj, jobjectArray jpfd)
{
  int pfd[2];
  int status;

  /* get C values from Java params */
  jobject jin = (*env) -> GetObjectArrayElement(env, jpfd, 0);
  jobject jout = (*env) -> GetObjectArrayElement(env, jpfd, 1);

  /* get file descriptor Java id for jin */
  jclass class = (*env) -> GetObjectClass(env, jin);
  jfieldID jinid = (*env) -> GetFieldID(env, class, "fd", "I");

  /* get file descriptor Java id for jout */
  jfieldID joutid;
  class = (*env) -> GetObjectClass(env, jout);
  joutid = (*env) -> GetFieldID(env, class, "fd", "I");


# ifdef DEBUG
    fprintf(stderr, "attempting to open pipe\n");
# endif

  /* process */
  status = pipe(pfd);
# ifdef DEBUG
    fprintf(stderr, "pipe in: %d, out: %d\n", pfd[0], pfd[1]);
# endif

  if (status < 0) {
    /* cleanup */

    /* set POSIX errno, throw exception? */
    throwPosixException(env, errno, strerror(errno));
    return;
  }

  /* set Java field values */
  (*env) -> SetIntField(env, jin, jinid, pfd[0]);
  (*env) -> SetIntField(env, jout, joutid, pfd[1]);

  /* cleanup */


  return((jint) status);

}
