/**
 * posix_Process.
 *
 * @version     1.0 20/6/98
 * @author      Jan Newmarch.
 * Copyright &copy; Jan Newmarch.
 * Covered by GNU Library General Public License
 */


#include <stdio.h>
#include <unistd.h>
#include "posix_Process.h"
#include "PosixException.h"

/*
 * Class:     posix_Process
 * Method:    fork
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_posix_Process_fork
  (JNIEnv *env, jclass class)
{
  int status;

  /* process */
  status = fork();

# ifdef DEBUG
    if (status == -1)
      throwPosixException(env, errno, strerror(errno));
# endif

  /* set error? */
  if (status < 0) {
    /* set error */
  }
  return((jint) status);
}

/*
 * Class:     posix_Process
 * Method:    execvp
 * Signature: (Ljava/lang/String;[Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_posix_Process_execvp
  (JNIEnv *env, jclass class, jstring jfile, jobjectArray jargv)
{
  int status;
  int n;

  /* get C values from Java params */
  const char *file = (*env) -> GetStringUTFChars(env, jfile, 0);
  jsize len = (*env) -> GetArrayLength(env, jargv);
  /* populate our own argv array */
  char **argv = (char **) malloc((len+1) * sizeof(char *));
  if (argv== NULL) {
    throwPosixException(env, errno, strerror(errno));
  }
  for (n = 0; n < len; n++) {
    jstring jstr = (jstring) (*env) -> GetObjectArrayElement(env, jargv, n);
    argv[n] = (*env) -> GetStringUTFChars(env, jstr, 0);
  }
  argv[n] = NULL;

# ifdef DEBUG
    fprintf(stderr, "attempting to execvp %s\n", file);
# endif

  /* process */
  status = execvp(file, argv);

# ifdef DEBUG
    if (status == -1)
      perror("");
# endif

  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jfile, file);

  /* set error? */
  if (status < 0) {
    /* set error */
    throwPosixException(env, errno, strerror(errno));
  }
  return((jint) status);

}



/*
 * Class:     posix_File
 * Method:    umask
 * Signature: (I)I
 */
/* NOTUSED yet */
JNIEXPORT jint JNICALL Java_posix_File_umask(JNIEnv *env,
					    jclass class,
					    jint jmask)
{
  /* get C values from Java params */
  mode_t mask = (mode_t) jmask;

# ifdef DEBUG
    fprintf(stderr, "setting umask to %o\n", jmask);
# endif

  /* process */
  mask = umask(mask);

  /* return value */
  return ((jint) mask);
}
