/**
 * posix_ReadFile.
 *
 * @version     1.0 20/6/98
 * @author      Jan Newmarch.
 * Copyright &copy; Jan Newmarch.
 * Covered by GNU Library General Public License
 */


#include "posix_ReadFile.h"
#include "PosixException.h"


/*
 * Class:     posix_ReadFile
 * Method:    read
 * Signature: ([BI)I
 */
JNIEXPORT jint JNICALL Java_posix_ReadFile_read___3BI(
					   JNIEnv *env,
                                           jobject obj, 
                                           jbyteArray jbuf,
                                           jint nbyte)
{
  int nread;
  jbyte *buf;

  /* get file descriptor Java id */
  jclass class = (*env) -> GetObjectClass(env, obj);
  jfieldID jfid = (*env) -> GetFieldID(env, class, "fd", "I");

  /* get file descriptor */
  int fd = (int) (*env) -> GetIntField(env, obj, jfid);

  /* get a native pointer to the array elmnts */
  buf = (*env) -> GetByteArrayElements(env, jbuf, NULL);
  
# ifdef DEBUG
    fprintf(stderr, "reading %u bytes from file descriptor %d\n", nbyt
e, fd);
# endif

  /* process */
  nread = read(fd, buf, nbyte);

# ifdef DEBUG
    fprintf(stderr, "read %d bytes from file descriptor %d\n", nread, 
fd);
# endif

  /* reset array elements */
  (*env) -> ReleaseByteArrayElements(env, jbuf, buf, 0);
  
  /* return value */
  if (nread == -1) {
    /* error */
    throwPosixException(env, errno, strerror(errno));
  }
  return (jint) nread;
}

