/**
 * posix_Stat.
 *
 * @version     1.0 20/6/98
 * @author      Jan Newmarch.
 * Copyright &copy; Jan Newmarch.
 * Covered by GNU Library General Public License
 */

#include <sys/types.h>
#include <sys/stat.h>
#include "posix_Stat.h"
#include "Fields.h"

/*
 * Class:     posix_Stat
 * Method:    initializeConstants
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_posix_Stat_initializeConstants
  (JNIEnv *env, jclass cls)
{
  jfieldID fid;

  /* file permissions */
#ifdef DEBUG
  fprintf(stderr, "Setting Stat constants\n");
#endif
  setStaticInt(env, cls, "s_irwxu", S_IRWXU);
  setStaticInt(env, cls, "s_irusr", S_IRUSR);
  setStaticInt(env, cls, "s_iwusr", S_IWUSR);
  setStaticInt(env, cls, "s_ixusr", S_IXUSR);
  setStaticInt(env, cls, "s_irwxg", S_IRWXG);
  setStaticInt(env, cls, "s_irgrp", S_IRGRP);
  setStaticInt(env, cls, "s_iwgrp", S_IWGRP);
  setStaticInt(env, cls, "s_ixgrp", S_IXGRP);
  setStaticInt(env, cls, "s_irwxo", S_IRWXO);
  setStaticInt(env, cls, "s_iroth", S_IROTH);
  setStaticInt(env, cls, "s_iwoth", S_IWOTH);
  setStaticInt(env, cls, "s_ixoth", S_IXOTH);
  setStaticInt(env, cls, "s_isuid", S_ISUID);
  setStaticInt(env, cls, "s_isgid", S_ISGID);

#ifdef DEBUG
  fprintf(stderr, "Set constants\n");
#endif
}


/*
 * Class:     posix_Stat
 * Method:    setFields
 * Signature: (Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_posix_Stat_stat
  (JNIEnv *env, jobject obj, jstring jpath)
{
  int status;
  struct stat buf;

  /* get C values from Java params */
  const char *path = (*env) -> GetStringUTFChars(env, jpath, 0);

# ifdef DEBUG
    fprintf(stderr, "attempting to stat %s\n", path);
# endif

  /* process */
  status = stat(path, &buf);

  if (status < 0) {
    /* cleanup */
    (*env) -> ReleaseStringUTFChars(env, jpath, path);

    /* set POSIX errno, throw exception? */
    return((jint) status);
  }

  /* set Java field values */
  setIntField(env, obj, "st_mode", buf.st_mode);
  setIntField(env, obj, "st_ino", buf.st_ino);
  setIntField(env, obj, "st_dev", buf.st_dev);
  setIntField(env, obj, "st_nlink", buf.st_nlink);
  setIntField(env, obj, "st_uid", buf.st_uid);
  setIntField(env, obj, "st_gid", buf.st_gid);
  setIntField(env, obj, "st_size", buf.st_size);
  setIntField(env, obj, "st_atime", buf.st_atime);
  setIntField(env, obj, "st_mtime", buf.st_mtime);
  setIntField(env, obj, "st_ctime", buf.st_ctime);

  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jpath, path);

  return((jint) status);

}

/*
 * Class:     posix_Stat
 * Method:    isDirectory
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_posix_Stat_isDirectory
  (JNIEnv *env, jobject obj)
{
  // is it quicker doing it through cached Java values
  // or by calling through stat() each time? Don't know...
  int mode = getIntField(env, obj, "st_mode");
  return S_ISDIR(mode);
}

/*
 * Class:     posix_Stat
 * Method:    isCharacterSpecial
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_posix_Stat_isCharacterSpecial
  (JNIEnv *env, jobject obj)
{
  int mode = getIntField(env, obj, "st_mode");
  return S_ISCHR(mode);
}

/*
 * Class:     posix_Stat
 * Method:    isBlockSpecial
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_posix_Stat_isBlockSpecial
  (JNIEnv *env, jobject obj)
{
  int mode = getIntField(env, obj, "st_mode");
  return S_ISBLK(mode);
}
/*
 * Class:     posix_Stat
 * Method:    isRegularFile
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_posix_Stat_isRegularFile
  (JNIEnv *env, jobject obj)
{
  int mode = getIntField(env, obj, "st_mode");
  return S_ISREG(mode);
}
/*
 * Class:     posix_Stat
 * Method:    isFIFO
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_posix_Stat_isFIFO
  (JNIEnv *env, jobject obj)
{
  int mode = getIntField(env, obj, "st_mode");
  return S_ISFIFO(mode);
}



