/**
 * posix_WriteFile.
 *
 * @version     1.0 20/6/98
 * @author      Jan Newmarch.
 * Copyright &copy; Jan Newmarch.
 * Covered by GNU Library General Public License
 */


#include "posix_WriteFile.h"
#include "PosixException.h"

/* #define DEBUG */



/*
 * Class:     posix_WriteFile
 * Method:    write
 * Signature: ([B)I
 */

JNIEXPORT jint JNICALL Java_posix_WriteFile_write___3BI(
						 JNIEnv *env,
                                                 jobject obj, 
                                                 jbyteArray jbuf,
                                                 jint nbyte)
{
  int nwrote;
  jbyte *buf;

  /* get file descriptor Java id */
  jclass class = (*env) -> GetObjectClass(env, obj);
  jfieldID jfid = (*env) -> GetFieldID(env, class, "fd", "I");

  /* get file descriptor */
  int fd = (int) (*env) -> GetIntField(env, obj, jfid);

  /* get a native pointer to the array elmnts */
  buf = (*env) -> GetByteArrayElements(env, jbuf, NULL);
  
# ifdef DEBUG
    fprintf(stderr, "writing %u bytes to file descriptor %d\n", nbyte,
 fd);
# endif

  /* process */
  nwrote = write(fd, buf, nbyte);

# ifdef DEBUG
    fprintf(stderr, "wrote %d bytes to file descriptor %d\n", nwrote, fd);
# endif

  /* reset array elements */
  (*env) -> ReleaseByteArrayElements(env, jbuf, buf, 0);
  
  /* return value */
  if (nwrote == -1) {
    /* error */
    throwPosixException(env, errno, strerror(errno));
  }
}

