/**
 * Posix test class TestOpenFile.
 *
 * @version     1.0 20/6/98
 * @author      Jan Newmarch.
 * Copyright &copy; Jan Newmarch.
 * Covered by GNU Library General Public License
 */

import posix.ReadFile;
import posix.WriteFile;
import posix.Posix;
import java.lang.Runtime;
import java.lang.Process;

/**
 * Test OpenFile and its subclasses.
 * Tests ReadFile, WriteFile constructors,
 * read, write, close.
 * Requires files/f1, files/f2.
 */
public class TestOpenFile {

    static public void main(String argv[]) {
        System.loadLibrary("java_posix");
        new TestOpenFile();
    }

    public TestOpenFile() {
	ReadFile in = null;
	WriteFile out = null;
	boolean error = false;

	try {
	    in = new ReadFile("files/f1");
	    out = new WriteFile("files/f2");
	} catch(Exception e) {
	    System.out.println("Couldn't open files. " +
			       e.toString());
	    error = true;
	}
	
	byte buf[] = new byte[1024];
	int nread = 0;

	try {
	    while ((nread = in.read(buf)) > 0) {
		out.write(buf, nread);
	    }
	} catch(Exception e) {
	    System.out.println("Couldn't read/write. " +
			       e.toString());
	    error = true;
	}

	try {
	    in.close();
	    out.close();
	} catch(Exception e) {
	    System.out.println("Couldn't close files. " +
			       e.toString());
	    error = true;
	}

	// see if the two are the same
	try {
	    int code;
	    Process p = Runtime.getRuntime().exec(
				 "cmp files/f1 files/f2");
	    // p.waitFor();
	    if ((code = p.waitFor()) != 0) {
		System.out.println("Copy using read/write failed, exit code " + code);
		error = true;
	    }
	} catch(Exception e) {
	    System.out.println("Couldn't create check process " +
			       e.toString());
	    error = true;
	}

	// now test stdin/stdout
	System.out.println("Type upto 128 chars");
	try {
	    nread = Posix.stdin.read(buf);
	} catch(Exception e) {
	    System.out.println("Couldn't read stdin " +
			       e.toString());
	    error = true;
	}
	System.out.println("This should be what you typed:");
	try {
	    Posix.stdout.write(buf, nread);
	} catch(Exception e) {
	    System.out.println("Couldn't write stdout " +
			       e.toString());
	    error = true;
	}

	if (error) {
	    System.out.println("OpenFile: tests failed");
        } else {
            System.out.println("OpenFile: tests passed");
        }

    }
}
