/**
 * Posix test class TestStat.
 *
 * @version     1.0 20/6/98
 * @author      Jan Newmarch.
 * Copyright &copy; Jan Newmarch.
 * Covered by GNU Library General Public License
 */

import posix.Stat;

/**
 * Test Stat class.
 * Tests isDirectory, getMode, 
 * isCharacterSpecial, isBlockSpecial.
 * Requires ., /dev/tty, /dev/hda
 */
public class TestStat {

    static public void main(String argv[]) {
	System.loadLibrary("java_posix");
        new TestStat();
    }

    public TestStat() {
	boolean error = false;
	Stat stat = null;

	try {
	    stat = new Stat(".");
	} catch(Exception e) {
	    System.out.println("Stat failed on .");
	    stat = null;
	    error = true;
	}

	if (stat != null) {
	    if( ! stat.isDirectory()) {
		System.out.println("Test failed: . is not a dir");
		error = true;
	    }

	    int mode = stat.getMode();
	    if ((mode & (Stat.RWXU | Stat.RWXG | Stat.RWXO)) !=
		(Stat.RUSR | Stat.WUSR | Stat.XUSR | Stat.RGRP |
		 Stat.XGRP | Stat.ROTH | Stat.XOTH)) {
		System.out.println("Failed mode constants: . has mode" +
				   mode);
		error = true;
	    }
	}

	try {
	    stat = new Stat("/dev/tty");
	} catch(Exception e) {
	    System.out.println("Stat failed on /dev/tty");
	    stat = null;
	    error = true;
	}
	if (stat != null && !stat.isCharacterSpecial())
		System.out.println("Failed: /dev/tty is not a character device");

	try {
	    stat = new Stat("/dev/hda");
	} catch(Exception e) {
	    System.out.println("Stat failed on /dev/hda");
	    stat = null;
	    error = true;
	}
	if (stat != null && !stat.isBlockSpecial()) {
	    System.out.println("Failed: /dev/hda is not a block device");
	    error = true;
	}

	if (error) {
	    System.out.println("Stat: tests failed");
	} else {
	    System.out.println("Stat: tests passed");
	}

    }
}





